function vol = getVolume(aligned)
% getVolume takes labelled cell positions and calculates a volume of the
% structure
%
% Inputs:
%   aligned: data structure containing all labelled cells from *ONE*
%   injection
%
% [vol] = getVolume(...) is the volume of the structure in mm^3
%
% Marissa Applegate, Aronov Lab 2022
bw = .1;

isoLevel = 0.5;
smooth = 0.5;

jitter = rand(size(aligned,1),1)/10-.05;
aligned.AP=aligned.AP+jitter; %add randomJitter to the AP position

lims = [min(aligned.AP(:,1))-1, min(aligned.ML)-1, min(aligned.DV)-1;...
    max(aligned.AP)+1,max(aligned.ML)+1, max(aligned.DV)+1];

aligned = table2array(aligned);

binnedCells = histcn(aligned,lims(1,1):bw:lims(2,1),lims(1,2):bw:lims(2,2),...
    lims(1,3):bw:lims(2,3));
binnedCells = imgaussfilt3(binnedCells,smooth);

vol = sum(binnedCells(:)>=isoLevel) ./ 1000;

