function figure2B(anatomy)
% generate figure2B 
%
%   anatomy: the anatomical data structure

% Marissa Applegate, Aronov Lab 2022

% find the best fit line for DL/CDL using 1000 points from each injection;
DL_CDL = zeros(0,3);
maxN = 1000;
for i = 1:numel(anatomy)
    d = anatomy(i).data;  
   for hemi = 1:2
       if hemi == 1
            mask = d.ROI == 'DL/CDL' & d.Transverse == 'Hp' & d.Hemisphere == 'L';
       else   
            mask = d.ROI == 'DL/CDL' & d.Transverse == 'Hp' & d.Hemisphere == 'R';
       end
    aligned = d(mask,:);
    inj = unique(aligned.Long); 
    for j = 1:length(inj)
        mask = aligned.Long==inj(j);
        a = aligned(mask,:); 
        
        % take up to 1000 data points
    if size(a,1)>=maxN
        ind = randperm(size(a,1),maxN);
    else
        ind = 1:size(a,1);
    end
        DL_CDL = [DL_CDL; table2array(a(ind,[2 5 6]))];
        
    end
   end
end
%%
f = fitlm(DL_CDL(:,3),DL_CDL(:,2));
slope = f.Coefficients{2,1};
itr = f.Coefficients{1,1};
v = [slope 1]; v = v/norm(v);

%% collect all of the DL/CDL data into a cell
allInj = nan(8,4);
allInjCell = cell(8,4);
for i = 3:numel(anatomy)
   d = anatomy(i).data;  
   for hemi = 1:2
       if hemi == 1
            mask = d.ROI == 'DL/CDL' & d.Transverse == 'Hp' & d.Hemisphere == 'L';
       else     
            mask = d.ROI == 'DL/CDL' & d.Transverse == 'Hp' & d.Hemisphere == 'R';
       end

    aligned = d(mask,:);
    inj = unique(aligned.Long); 
    
    for j = 1:length(inj)
        mask = aligned.Long==inj(j);
        a = aligned(mask,:); 
  
        %find Next available
        injNum = (inj(j)+1)/2 +1;
        opt = allInj(:,injNum);
        nextAvail = cumsum(~isnan(opt));
        nextAvail = nextAvail(end)+1;
        
        allInjCell{nextAvail,injNum} = table2array([a(:,6) a(:,5)]);  
        allInj(nextAvail,injNum) = 1;
    end
   end
end
%%
% plot the violin plots
close all
% order injections arbitratily (no inherernt order initially)
incl = cell(4,1);
incl{1} = [1 3 2 4];
incl{2} = [1 3 2 4];
incl{3} = [1 6 2 5 4 3];
incl{4} = [1 4 2 3 5 6];

clc
c = 0;
cols = [152 78 163;62 55 144; 73 174 74; 163 18 1]/255;
hold on

inj = [-1 1 3 5 0]';
ML = (inj-itr)/slope;
injLoc = [ML inj] *v';

for loc = 1:4
      hold on
      for i = incl{loc}
         pts = allInjCell{i,loc}*v' - injLoc(end);
        addViolin(pts,c/2 ,cols(loc,:),.1)  
          c = c+1;
      end
          
c = c+2;
end

off = [.75 3.75 7.25 11.25];
set(gca,'xtick',off,'xTickLabel',{'-1A','1A','3A','5A'})
xlim([-.5 13])
ylabel({'long axis', 'coordinate (mm)'})
xlabel('injection location')
    

function addViolin(p,x,c,den)

for m = 1:length(x)
    hMin = min(p(:,m));
    hMax = max(p(:,m));
    incl = p(:,m)>hMin & p(:,m)<hMax;

    [h,range] = ksdensity(p(incl,m),hMin-den/2*5:den:hMax+den/2*5);

    h = h/max(h(:)) /10;
    
    nBin = length(range); 
    revOrder = nBin-(0:nBin-1);
    range = range(:);
    h = h(:);
    patch([h + x(m); x(m)-h(revOrder)],[range; range(revOrder)],c(m,:), 'edgeColor', 'none')

end

