function DsampCNMFE(varargin)
% takes in folder of mats, and NBorderCutPixels for motion correction
% performs downsampled CNMFE
close all; 
%% choose folder 
engramPath = 'Z:\Marissa\openField'; 

% Choose bird and date (via selection from Engram MISCToDownsample folder)
if length(varargin)==0
    [FolderOfMats] = uigetdir([engramPath],....
        'Select folder of mats');
else
    FolderOfMats = varargin{1}; 
end
str = split(FolderOfMats,'\');
BIRD = str{end-2}; 
DATE = str{end-1}; 
display(['Preparing: ' BIRD ' ' DATE])

%% complile list of files in folder
DIR = dir(fullfile(FolderOfMats, ['tFile*Startframe*.mat']));

%% pick cropping window for motion selection
XminmaxYminmax = findMotionCrop(BIRD);
FiltSD = 5; % check ref image if changing
%% Calculate DsampY and xyshifts, based on file list.
% initialize
ResultsFolder = fullfile(FolderOfMats, 'Results'); 
mkdir(ResultsFolder)
DsampY = []; 
ThisFramesFile = []; 
ThisFramesFrameWithinFile = []; 
FrameFileStarts = []; 
FrameFileEnds = []; 
NumPrevFrames = 0; 
FrameFileStarts_DS = []; 
FrameFileEnds_DS = []; 
NumPrevFrames_DS = 0; 
xshifts = {}; 
yshifts = {}; 


% set up variables
load(fullfile(FolderOfMats, DIR(1).name), 'Ysiz'); 
d1 = Ysiz(1);
d2 = Ysiz(2);
Fs = 20;
DsampFac = 1; 
if length(varargin)>1
    NBorderCutPixels = varargin{2}; 
else
    if  isequal(BIRD,'ROS40')
        NBorderCutPixels = 3; 
    else
        NBorderCutPixels = 2; 
    end
end
snMat = zeros((d1-2*NBorderCutPixels)*(d2-2*NBorderCutPixels),length(DIR)); % first and last 2 rows and columns cropped after motion correction

% create reference frame by filtering some data from first file. 
load(fullfile(FolderOfMats, DIR(1).name)); 

%filter for motion correction
FiltY = double(Y); 

for fi = 1:Ysiz(3)
    FiltY(:,:,fi) = double(Y(:,:,fi)) - imgaussfilt(double(Y(:,:,fi)),FiltSD); %  imfilter(single(Y(:,:,fi)), kernel1);
end
refframe = mean(FiltY,3); % initial reference frame

% calculate shifts and motion correct
[xshiftsPrelim, yshiftsPrelim] = track_subpixel_wholeframe_motion_fft_forloop(...
    FiltY(XminmaxYminmax(3):XminmaxYminmax(4), XminmaxYminmax(1):XminmaxYminmax(2),:),...
    refframe(XminmaxYminmax(3):XminmaxYminmax(4), XminmaxYminmax(1):XminmaxYminmax(2)));%ToCorrect, refframe);
fillval = nan;
mov = translateAcq(Y, xshiftsPrelim, yshiftsPrelim, fillval);
refframe = mean(mov,3) - imgaussfilt(mean(mov,3),FiltSD); % better, motion-corrected, reference frame, to use on whole dataset
imagesc(refframe(XminmaxYminmax(3):XminmaxYminmax(4), XminmaxYminmax(1):XminmaxYminmax(2))); title('Reference for motion correction')




for filei = 1:length(DIR)
    % load file
    load(fullfile(FolderOfMats, DIR(filei).name)); 
    display(['Loaded file ' num2str(filei) '/' num2str(length(DIR))])
    FrameFileStarts(filei) = NumPrevFrames+1; 
    FrameFileEnds(filei) = NumPrevFrames+Ysiz(3); 
    FrameFileStarts_DS(filei) = NumPrevFrames_DS+1;
    ThisFramesFile = [ThisFramesFile filei*ones(1,Ysiz(3))]; 
    ThisFramesFrameWithinFile = [ThisFramesFile 1:Ysiz(3)]; 
    NumPrevFrames = NumPrevFrames+Ysiz(3);
    


    % filter for motion correction
    FiltY = double(Y); 
    for fi = 1:Ysiz(3)
        FiltY(:,:,fi) = double(Y(:,:,fi)) - imgaussfilt(double(Y(:,:,fi)),FiltSD); %  imfilter(double(Y(:,:,fi)), kernel1);
    end

    % calculate shifts and motion correct
    [xshifts{filei}, yshifts{filei}] = track_subpixel_wholeframe_motion_fft_forloop(...
        FiltY(XminmaxYminmax(3):XminmaxYminmax(4), XminmaxYminmax(1):XminmaxYminmax(2),:),...
        refframe(XminmaxYminmax(3):XminmaxYminmax(4), XminmaxYminmax(1):XminmaxYminmax(2)));%ToCorrect, refframe);
    fillval = nan; %mean(Y(:));
    mov = translateAcq(Y, xshifts{filei}, yshifts{filei}, fillval);
    display(['Corrected motion file ' num2str(filei) ...
        ', max shift: ' num2str(max([xshifts{filei}(:); yshifts{filei}(:)])) ' pixels'])
    
    if filei == 1 % save for diagnostic movie
        Diagnostics.Y_raw = Y((1+NBorderCutPixels):(end-NBorderCutPixels), (1+NBorderCutPixels):(end-NBorderCutPixels),:); 
        Diagnostics.Y_motion_correct = mov((1+NBorderCutPixels):(end-NBorderCutPixels), (1+NBorderCutPixels):(end-NBorderCutPixels),:); 
    end
    
    % define new Y and Ysiz, cutting off borders 
    Y = mov((1+NBorderCutPixels):(end-NBorderCutPixels), (1+NBorderCutPixels):(end-NBorderCutPixels),:); clear mov
    figure(1); clf; imagesc(Y(:,:,1)); title(['Example frame file ' num2str(filei) '/' num2str(length(DIR))]); drawnow
    Ysiz = size(Y); 
    d1 = Ysiz(1);
    d2 = Ysiz(2);
    Y = reshape(Y,d1*d2,[]);
    
    % Compute sn
    UseFrams = 100+(1:1000); % use a short clip from each file to estimate sn.
    UseFrams(UseFrams>size(Y,2))=[];
    for pxi = 1:size(Y,1)
        [~, snMat(pxi,filei)] = estimate_baseline_noise(double(Y(pxi,UseFrams)));
    end
    display('Computed sn')
    
    % Downsample data in time
    fstart = 1; 
    while fstart<=(size(Y,2)-DsampFac*Fs)
        DsampY(:,NumPrevFrames_DS+1) = mean(double(Y(:,fstart:min(size(Y,2), fstart+DsampFac*Fs-1))),2); 
        fstart = fstart+DsampFac*Fs;
        NumPrevFrames_DS = NumPrevFrames_DS+1; 
    end
    FrameFileEnds_DS(filei) = size(DsampY,2); 
    display('Stored in DSampY')
end
%% average sn across files
sn = mean(snMat,2); 
figure(1); clf; imagesc(reshape(sn,d1,d2));
title('sn combined'); drawnow; shg

%% save some diagnostics
Diagnostics.xshifts = xshifts; Diagnostics.yshifts = yshifts; 
Diagnostics.sn = sn; 

%% do CNMFE on downsampled data

%  create Source2D class object for storing results and parameters
min_corr = .7; 
min_pnr = min_corr*10; 
neuron = Sources2D(...
    'gSig',4,...                                  % sigma of the 2D gaussian (width of the gaussian kernel) that approximates cell bodies
    'gSiz',10,...                                 % 0.5*window width of cropping for initiation
    'min_corr',min_corr,'min_pnr',min_pnr, ...               % ELM was 'min_corr',0.9,'min_pnr',5 % changed from .8 5
    'center_psf',true, ...
    'min_pixel',50,...                            % minimum number of nonzero pixels for each neuron
    'max_pixel',800,...                           % maximum number of nonzero pixels for each neuron
    'd1', Ysiz(1), ...
    'd2', Ysiz(2), ...
    'bd',0,...                                    % number of rows/columns to be ignored in the boundary (mainly for motion corrected data)
    'background_model', 'ring');    
neuron.Fs = 20;                               % frame rate  
% options for running deconvolution
neuron.options.deconv_flag=false;             % within cnmf_e, set neuron.options.deconv_flag to false as it might introduce error
neuron.options.deconv_options = struct('type', 'ar2', ... % model of the calcium traces. {'ar1', 'ar2'}
    'method', 'constrained', ... % method for running deconvolution {'foopsi', 'constrained', 'thresholded'}
    'optimize_pars', true, ...   % optimize AR coefficients
    'optimize_b', false, ...      % optimize the baseline
    'optimize_smin', false);      % optimize the threshold
bg_neuron_ratio = 1.1;   % spatial range / diameter of neurons
Y = DsampY;    
% central limit theorem to calculate what sn to expect on downsampled data: 
neuron.P.sn = sn./sqrt(DsampFac*Fs); 

% detrend data
NumMinutes = size(Y,2)/60;
nk = floor(NumMinutes); 
detrend_method = 'spline';
[Ydt, X_todetrend, R_todetrend] = detrend_data(Y, nk, detrend_method); 

% fit local background
rr = ceil(neuron.options.gSiz * bg_neuron_ratio); 
[Ybg, results] = local_background(reshape(Ydt,d1,d2,[]), 1, rr, [], reshape(neuron.P.sn,d1,d2,[]), []);
Ybg = reshape(Ybg, d1*d2,[]);
weights = results.weights; 
Ysignal = Ydt - Ybg; 

% if want to look at corr image and pnr to judge params
[Cn,pnr] = neuron.correlation_pnr(Ysignal); 
figure(6); imagesc(pnr); colorbar; title('pnr'); 
figure(7); imagesc(Cn); colorbar; title('Cn'); 
figure(8); imagesc((pnr>min_pnr)|(Cn>min_corr)); title('pnr or cn above thres'); 
Diagnostics.Cn = Cn; Diagnostics.pnr = pnr; 
drawnow

% initialize neuron locations
[results, center, Cn, PNR] = greedyROI_endoscope(Ysignal, [], neuron.options);%, debug_on, save_avi);
neuron.C = results.Cin;
neuron.A = results.Ain; 
neuron.C_raw = results.Cin_raw; 

% update A and C
for iteri = 1:4
    neuron.updateSpatial_endoscope(Ysignal, [], 'nnls', [], false);
    neuron.updateTemporal_endoscope(Ysignal);
    neuron.quickMerge([.1 .2 nan]); % thresA, thresC, thresS... merged if corr is above thresA&C (S is placeholder)
end
neuron.updateSpatial_endoscope(Ysignal, [], 'nnls', [], false);
neuron.updateTemporal_endoscope(Ysignal);
%% Plot CNMFE As

% plot the neuron locations
figure(1); clf;
ColorAllNeurons(neuron.A, Ysiz(1),Ysiz(2));
title('Preliminary A')
drawnow

%% get A's and C's file by file
Call = []; 
for filei = 1:1%length(DIR)
    
    % Load file
    load(fullfile(FolderOfMats, DIR(filei).name)); 

    % apply motion correction, and define new Y and Ysiz, cutting off borders 
    mov = translateAcq(Y, xshifts{filei}, yshifts{filei}, fillval);
    Y = mov((1+NBorderCutPixels):(end-NBorderCutPixels), (1+NBorderCutPixels):(end-NBorderCutPixels),:); clear mov
    Ysiz = size(Y); 
    d1 = Ysiz(1);
    d2 = Ysiz(2);
    Y = reshape(Y,d1*d2,[]);
    display(['Loaded file ' num2str(filei) ', corrected motion'])

    % Detrend the original Y
    Yorig = Y; 
    ind_DS_file = FrameFileStarts_DS(filei):FrameFileEnds_DS(filei); 
    Xorig = interp1(linspace(0, size(Yorig,2), length(ind_DS_file)),full(X_todetrend(ind_DS_file,:)), 1:size(Yorig,2));
    Yorig_dt = double(Yorig) - double(R_todetrend*Xorig');
    
    if filei == 1 % save for diagnostic movie
        Diagnostics.Y_detrended = reshape(Yorig_dt,d1,d2,[]); 
    end

    % clear unnecessary big variables to help with memory demands
    clear Y Ybg Ydt Ysignal FiltY 
    
    % reconstruct the original BG
    tic; 
    Yorig_bg = double(zeros(size(Yorig_dt)));
    for m=1:size(Yorig_dt,1)
        w = (weights{m});
        Yorig_bg(m,:) = w(2,:)*(Yorig_dt(w(1,:),:));
    end
    display(['Computed background'])
    toc

    % compile original BG subtracted signal, and initial C
    Yorig_signal = (double(Yorig_dt)-Yorig_bg); 
   % Yorig_signal(Yorig_signal<0) = 0; 
    Cin_orig = zeros(size(neuron.C,1), size(Yorig_dt,2));
    neuron.C = Cin_orig; % not actually used (gets zeroed out; size of variable is all that matters)
    
    if filei == 1 % save for diagnostic movie
        Diagnostics.Y_background_subtracted = reshape(Yorig_signal,d1,d2,[]); 
    end
    
    % update C
    neuron.updateTemporal_endoscope(Yorig_signal);
    Call(:,end+(1:size(neuron.C,2))) = neuron.C; 
    
end
neuron.C = Call; 
neuron.C_raw = Call; 
neuron.S = 0*Call; 
%% unpack variables
C = neuron.C; 
C_raw = neuron.C_raw; 
S = neuron.S; 
A = neuron.A; 

%% deconvolve
[C,S] = deconvolve_ELM(C_raw,Fs);

%% save post deconv
save(fullfile(ResultsFolder, ['ExtractedPostDeconv_' datestr(now, 'ddmmyyyy_HHMMSS') '.mat']), ...
    'C', 'C_raw', 'S', 'A', 'd1', 'd2', ...
    'FolderOfMats', 'FrameFileEnds', 'FrameFileStarts', 'ThisFramesFile', 'ThisFramesFrameWithinFile', 'Fs',...
    '-v7.3')

