function B = processSpeed(B, fps, tsmooth_linear_speed, speed_thresh, max_duration)
% from Payne 2021

% Get the linear speed of the head (in mm/s) projected on xy plane
nsmooth_linear_speed = round(tsmooth_linear_speed*fps/2)*2+1;
smooth_xx = smooth(B.xx, nsmooth_linear_speed, 'moving');
smooth_yy = smooth(B.yy, nsmooth_linear_speed, 'moving');
B.speed = [sqrt(diff(smooth_xx).^2+diff(smooth_yy).^2)*fps; NaN];
clear smooth_xx smooth_yy;

% Create mask for speed thresholding
if speed_thresh
    if max_duration
        code_area = regionprops(B.speed<speed_thresh,'area');
        code_label = bwlabel(B.speed<speed_thresh);
        stationary_lengths = [code_area.Area]/fps;
        reject_segs = find(stationary_lengths > max_duration);
        B.speed_mask = false(size(B.xx));
        for jj = 1:length(reject_segs)
            B.speed_mask(code_label==reject_segs(jj)) = 1;
        end
    else
        B.speed_mask = B.speed < speed_thresh;
    end
    
end

% Apply the speed thresholding to the output
B.speed_raw = B.speed;
B.xx(B.speed_mask) = NaN;
B.yy(B.speed_mask) = NaN;
B.speed(B.speed_mask) = NaN;
if isfield(B,'head_angle')
    B.head_angle(B.speed_mask) = NaN;
end
