function [Tuning,Parameter] = calculateAllTuningCurves(rawXX, rawYY, headAngle, S)
% for 1 cell, take the input of the behavioral and calcium data from one 
% cell and calculate all tuning curves and parameters

%% preprocessing for speed thresholding
B.xx = rawXX;
B.yy = rawYY;
B = processSpeed(B, 20, 1, 50, 5);

xx = B.xx; 
yy = B.yy;

speed = B.speed';
headAngle(B.speed_mask)=nan;

% calculate the spatial tuning
[map, normInfo, pSpatial, timeShiftedInfo, threshMap, BS, pBS] ...
    = calculateSpatialTuning(xx, yy, S);

% calculate spatial stability
[xcorr_val, sigStab] = calculateStability(xx, yy, S);

% calculate head direction tuning
[HDTuning,MVL,sigHD] = calculateHDTuning(headAngle, S);

% calculate the speed tuning
[speedTuning,speedScore, sigSpeed] = calculateSpeedTuning(speed,S);

% calculate the dash modulation
[dashTuning, dashScore, sigDash] = calculateDashModulation(speed,S);

Tuning = struct;
Parameter = struct;

% save the parameters
Tuning.spatialMap = map;
Tuning.fieldSegmentation = threshMap;
Tuning.temporalShift = timeShiftedInfo;
Tuning.headDirectionTuning = HDTuning;
Tuning.speedTuning = speedTuning;
Tuning.dashTuning = dashTuning;

Parameter.normSpatialInfo = normInfo;
Parameter.pSpatial = pSpatial;
Parameter.stability = xcorr_val;
Parameter.pStability = sigStab;
Parameter.borderScore = BS;
Parameter.pBorder = pBS;
Parameter.headDirectionScore = MVL;
Parameter.pHeadDirection = sigHD;
Parameter.speedScore = speedScore;
Parameter.pSpeed = sigSpeed;
Parameter.dashScore = dashScore;
Parameter.pDash =sigDash;