function [tuningCurve,speedScore, sig] = calculateSpeedTuning(speed,S)
% calculateSpeedTuning calculates the speed tuning curve, as well as speed
% score and the significance of the speed score for an individual cell
%
% speedTuning = calculateSpeedTuning(speed, S)
% calculates the speed tuning curve. Inputs:
%   speed: animal's speed in mm/s
%   S: Vector of spike amplitudes
%
% [speedTuning,speedScore] = calculateSpeedTuning(...) also returns the 
% speed score 
%
% [speedTuning,speedScore, sig] = calculateSpeedTuning(...) also returns the
% p value that the cell has significant speed tuning
%
% Marissa Applegate, Aronov Lab 2022

nshuff = 500;

%smooth speed and smooth firing rate
S = movmean(S,7);
speed = movmean(speed,7);

ST_coeff = getcc(S,speed);
bins = 0:5:200;
tuningCurve = calculateTuningCurve(speed,bins,S);
speedScore = getcc(tuningCurve,bins(1:end-1));

% shuffle randomly
randShifts = randi(length(speed),nshuff,1);
randcorr = zeros(size(randShifts));
for i = 1:nshuff
    randcorr(i) = getcc(S,[speed(randShifts(i):end) speed(1:randShifts(i)-1)]);
end
sig= sum(randcorr.^2>ST_coeff^2)/nshuff;

end

function cc = getcc(S,speed)
cc = corrcoef(S,speed,'rows','complete');
cc = cc(1,2);
end

function tuningCurve = calculateTuningCurve(speed,bins,S)
fps = 20;
[speedBins, ~, speedBinsIdentity] = histcounts(speed,bins);
%
tuningCurve = zeros(1,length(bins)-1);

for binID = 1:length(bins)-1
    mask = speedBinsIdentity == binID;
    A = sum(S(mask));
    B = speedBins(binID);
    tuningCurve(binID) =A/B;
end
tuningCurve = tuningCurve * fps;

end