function figure7a(data,condition)
% generate figure7a
%
%   data: the physiology data structure
%   condition: specify which of the recording conditions (locations) to
%   plot
% condition 1 == aDL
% condition 2 == pDL
% condition 3 == aHp
% Marissa Applegate, Aronov Lab 2022

% initialize vectors for accumulating the parameters
spatial = [];
spatialSigAll = [];

HD = [];
HDSig = [];

speedScore = [];
speedSig = [];

dashScoreAll = [];
dashScoreSig = [];


for b = 1:size(data,1)
    Parameter = data(b,condition).Parameter;

    % spatial
   info = vertcat(Parameter.normSpatialInfo);
   spat= vertcat(Parameter.pSpatial);
   stab = vertcat(Parameter.pStability);
   maskSpatial = spat<.01 & stab<.01;
   
   spatial= [spatial; info];
   spatialSigAll= [spatialSigAll; maskSpatial];
   
   % head direction   
   HD= [HD; vertcat(Parameter.headDirectionScore)];
   HDSig= [HDSig; vertcat(Parameter.pHeadDirection)<0.01];

   %speed
   speedScore = [speedScore; vertcat(Parameter.speedScore)];
   speedSig = [speedSig; vertcat(Parameter.pSpeed)<.01/2];


   % dash
   dashScoreAll = [dashScoreAll;vertcat(Parameter.dashScore)];
   dashScoreSig = [dashScoreSig; vertcat(Parameter.pDash)<.01];

end

%convert to logical
spatialSigAll = logical(spatialSigAll);
HDSig = logical(HDSig);
speedSig = logical(speedSig);
dashScoreSig = logical(dashScoreSig);


%% make the plots
cellSize = 1;
offset = 3;
aValue = .8;
lw = .2;

colors = [152 78 163; 55 126 184; 228 26 28; 164 164 164;]/255;
% spatial vs head direction
figure
c = zeros(length(HD),3);
for i = 1:3
c(HDSig& spatialSigAll,i) = colors(1,i);
c(HDSig& ~spatialSigAll,i) = colors(2,i);
c(~HDSig& spatialSigAll,i) = colors(3,i);
c(~HDSig& ~spatialSigAll,i) = colors(4,i);
end

scatter(spatial+offset,HD,cellSize,c,'MarkerFaceAlpha',...
    aValue,'MarkerEdgeAlpha',aValue,'lineWidth',lw)
set(gca,'xscale','log')
set(gca,'yscale','linear')
xlim([1 50])
ylim([0 .9])
ylabel('head direction score')
xlabel('spatial info (norm)')

%% spatial vs speed

figure
hold on
c = zeros(length(speedScore),3);
for i = 1:3
c(speedSig & spatialSigAll,i) = colors(1,i);
c(speedSig & ~spatialSigAll,i) = colors(2,i);
c(~speedSig & spatialSigAll,i) = colors(3,i);
c(~speedSig & ~spatialSigAll,i) = colors(4,i);
end

scatter(spatial+ offset,speedScore,cellSize,c,'MarkerFaceAlpha',...
    aValue,'MarkerEdgeAlpha',aValue,'lineWidth',lw)
set(gca,'yscale','linear')
set(gca,'xscale','log')
ylim([-1 1])
xlim([1 50])
ylabel('speed score')
xlabel('spatial info (norm)')

%% spatial vs dash
figure
hold on
c = zeros(length(speedScore),3);
for i = 1:3
c(dashScoreSig & spatialSigAll,i) = colors(1,i);
c(dashScoreSig & ~spatialSigAll,i) = colors(2,i);
c(~dashScoreSig & spatialSigAll,i) = colors(3,i);
c(~dashScoreSig & ~spatialSigAll,i) = colors(4,i);
end

scatter(spatial + offset,dashScoreAll,cellSize,c,'MarkerFaceAlpha',...
    aValue,'MarkerEdgeAlpha',aValue,'lineWidth',lw)
set(gca,'yscale','linear')
set(gca,'xscale','log')
ylim([.15 1])
xlim([1 50])
xlabel('spatial info (norm)')
ylabel('dash score')

%% speed vs dash
figure
hold on
c = zeros(length(speedScore),3);
for i = 1:3
c(speedSig & dashScoreSig,i) = colors(1,i);
c(~speedSig & dashScoreSig,i) = colors(2,i);
c(speedSig & ~dashScoreSig,i) = colors(3,i);
c(~speedSig & ~dashScoreSig,i) = colors(4,i);
end

scatter(speedScore,dashScoreAll,cellSize,c,'MarkerFaceAlpha',...
    aValue,'MarkerEdgeAlpha',aValue,'lineWidth',lw)
set(gca,'yscale','linear')
set(gca,'xscale','linear')
xlim([-1 1])
ylim([.15 1])
xlabel('speed score')
ylabel('dash score')

%% HD vs dash
figure
hold on
c = zeros(length(speedScore),3);
for i = 1:3
c(HDSig & dashScoreSig,i) = colors(1,i);
c(~HDSig & dashScoreSig,i) = colors(2,i);
c(HDSig & ~dashScoreSig,i) = colors(3,i);
c(~HDSig & ~dashScoreSig,i) = colors(4,i);
end

scatter(HD,dashScoreAll,cellSize,c,'MarkerFaceAlpha',...
    aValue,'MarkerEdgeAlpha',aValue,'lineWidth',lw)
set(gca,'xscale','linear')
set(gca,'yscale','linear')
xlim([0 .9])
ylim([.15 1])
ylabel('head direction score')
xlabel('dash score')

%% HD vs speed
figure
hold on
c = zeros(length(speedScore),3);
for i = 1:3
c(HDSig & speedSig,i) = colors(1,i);
c(~HDSig & speedSig,i) = colors(2,i);
c(HDSig & ~speedSig,i) = colors(3,i);
c(~HDSig & ~speedSig,i) = colors(4,i);
end

scatter(HD,speedScore,cellSize,c,'MarkerFaceAlpha',aValue,...
    'MarkerEdgeAlpha',aValue,'lineWidth',lw)
set(gca,'xscale','linear')
set(gca,'yscale','linear')
xlim([0 .9])
ylim([-1 1])
xlabel('head direction score')
ylabel('speed score')


