function [param_mean,fval] = fit_model(A,spiketrain,modelType,numFolds,initCon)

%% Description
% This code will section the data into 10 different portions. Each portion
% is drawn from across the entire recording session. It will then
% fit the model to 9 sections, and test the model performance on the
% remaining section. This procedure will be repeated 10 times, with all
% possible unique testing sections. The fraction of variance explained, the
% mean-squared error, the log-likelihood increase, and the mean square
% error will be computed for each test data set. In addition, the learned
% parameters will be recorded for each section.

% adapted from Hardcastle et al (2017) by Marissa Applegate

% Initialize matrices and section the data for k-fold cross-validation
[~,numCol] = size(A);
sections = numFolds*5;

% divide the data up into 5*num_folds pieces
edges = round(linspace(1,numel(spiketrain)+1,sections+1));

% initialize matrices
paramMat = nan(numFolds,numCol);
fval = nan(numFolds,1);
%% perform k-fold cross validation
for k = 1:numFolds
    %fprintf('\t\t- Cross validation fold %d of %d\n', k, numFolds);
    
    % get test data from edges - each test data chunk comes from entire session
    test_ind  = [edges(k):edges(k+1)-1 edges(k+numFolds):edges(k+numFolds+1)-1 ...
        edges(k+2*numFolds):edges(k+2*numFolds+1)-1 edges(k+3*numFolds):edges(k+3*numFolds+1)-1 ...
        edges(k+4*numFolds):edges(k+4*numFolds+1)-1]   ;
    
    test_spikes = spiketrain(test_ind); %test spiking
    test_A = A(test_ind,:); % testing behavior
    
    % training data
    train_ind = setdiff(1:numel(spiketrain),test_ind);
    train_spikes = spiketrain(train_ind);%training spikes
    train_A = A(train_ind,:);% training behavior
    
    %opts = optimset('Gradobj','on','Hessian','on','Display','off');
    opts = optimset('Display','off');

    data{1} = train_A; data{2} = train_spikes;

    if modelType(1)>0
    D1 = spdiags(ones(sqrt(modelType(1)),1)*[-1 1],0:1,sqrt(modelType(1))-1,sqrt(modelType(1)));
    DD1 = D1'*D1;
    M1 = kron(eye(sqrt(modelType(1))),DD1); M2 = kron(DD1,eye(sqrt(modelType(1))));
    M = (M1 + M2);  
    else
        M = [];
    end
    epsilon = .001;
    
    attempts = 1;
    
    
    while attempts < 10   
    if k == 1
        init_param = 1e-3*randn(numCol, 1) + initCon;%biasing initial 
    else
        init_param = param + 1e-3*randn(numCol, 1);
    end
        
    try
        [param,~,~] = fminunc(@(param) ln_poisson_mca(param,data,modelType,M,epsilon),init_param,opts);
        attempts = 10;
    catch
        disp(['failed convergence attempt ' num2str(attempts)])
        attempts = attempts+1;
    end
    end
    % compute llh increase from "mean firing rate model" 
    r = exp(test_A * param); 
    n = test_spikes; 

    llh = -sum(log(gampdf(n+epsilon,r+epsilon,1)));

    fval(k,:) = llh;
    
   % save the parameters
   paramMat(k,:) = param;
end
param_mean = nanmean(paramMat);

return
