function runAllModels(data,recordingLoc,bird,fp)
% data should be the physiology dataset of all recordings
% recordingLoc is the recording location. 1 == aDL, 2 == pDL, 3 == aHP
% bird is the bird or range of birds that should be run
% fp is the file path for where the model evaluations will be saved

numFolds = 10;%% for each neuron how many splits of the data

n_pos_bins = [3 7 11];
n_dir_bins = 18;
n_speed_bins = 10;
n_run_bins = 30; % 

% acceptedRanges
speedRange = [0 200];
speedRange = 0:speedRange(2)/n_speed_bins:speedRange(2);
HDRange = -pi:(2*pi)/n_dir_bins:pi;
spaceRange = [20 900];
modelAll = struct;
%
for b = bird
    sMax = numel(data(b,recordingLoc).sessions);
for s = 1:sMax

Behavior = data(b,recordingLoc).Behavior;
Calcium = data(b,recordingLoc).Calcium;
HD = Behavior(s).headAngle';

X = Behavior.xpos'; X(X<spaceRange(1))= spaceRange(1); X(X>spaceRange(2))=spaceRange(2);
Y = Behavior.ypos'; Y(Y<spaceRange(1))= spaceRange(1); Y(Y>spaceRange(2))=spaceRange(2);
speed = Behavior.speed';

% calculate runModulation time
runTimes = speedCrossingBin(speed,n_run_bins);

% Account for some missing frames of head direction data in some files
if length(HD)<length(X)
    HD = [HD;nan(length(X) - length(HD),1)];
elseif length(HD)>length(X)
    HD = HD(1:length(X));
end

maskOut = ~(isnan(X) | isnan(HD))& Behavior.speed_mask';
%
X = X(maskOut);
Y = Y(maskOut);
speed = speed(maskOut);
HD = HD(maskOut);
runTimes = runTimes(maskOut);

S = Calcium(s).S(:,maskOut);
% get positional xy bins
[~,~,~,pos1x,pos1y]= histcounts2(X,Y,linspace(spaceRange(1),spaceRange(2),n_pos_bins(1)+1),linspace(spaceRange(1),spaceRange(2),n_pos_bins(1)+1));
[~,~,~,pos2x,pos2y]= histcounts2(X,Y,linspace(spaceRange(1),spaceRange(2),n_pos_bins(2)+1),linspace(spaceRange(1),spaceRange(2),n_pos_bins(2)+1));
[~,~,~,pos3x,pos3y]= histcounts2(X,Y,linspace(spaceRange(1),spaceRange(2),n_pos_bins(3)+1),linspace(spaceRange(1),spaceRange(2),n_pos_bins(3)+1));

% convert xybin to actual bin
pos1Bin = pos1x + (pos1y-1).*n_pos_bins(1);
pos2Bin = pos2x + (pos2y-1).*n_pos_bins(2);
pos3Bin = pos3x + (pos3y-1).*n_pos_bins(3);

% get the HD, speed, and run bins
[~,~,speedBin]= histcounts(speed,speedRange);
[~,~,HDBin]= histcounts(HD,HDRange);
speedBin(speedBin==0) = n_speed_bins;
runTimes(runTimes==0)=n_run_bins;
%make the labels
binsAll = struct;

binsAll(1).label = 'Pos3';
binsAll(2).label = 'Pos7';
binsAll(3).label = 'Pos11';
binsAll(4).label = 'HD';
binsAll(5).label = 'Speed';
binsAll(6).label = 'Run';

% make the sparse matrices
binsAll(1).bins = zeros(length(X),n_pos_bins(1).^2);
binsAll(2).bins = zeros(length(X),n_pos_bins(2).^2);
binsAll(3).bins = zeros(length(X),n_pos_bins(3).^2);
binsAll(4).bins = zeros(length(X),n_dir_bins);
binsAll(5).bins = zeros(length(X),n_speed_bins);
binsAll(6).bins = zeros(length(X),n_run_bins);

% fill in the sparse matrices
r = 1:length(X);
binsAll(1).bins(sub2ind(size(binsAll(1).bins), (r(pos1y>0))',pos1Bin(pos1y>0))) = 1;
binsAll(2).bins(sub2ind(size(binsAll(2).bins), (r(pos2y>0))',pos2Bin(pos2y>0))) = 1;
binsAll(3).bins(sub2ind(size(binsAll(3).bins), (r(pos3y>0))',pos3Bin(pos3y>0))) = 1;
binsAll(4).bins(sub2ind(size(binsAll(4).bins), (r(HDBin>0))',HDBin(HDBin>0))) = 1;
binsAll(5).bins(sub2ind(size(binsAll(5).bins), (r)',speedBin)) = 1;
binsAll(6).bins(sub2ind(size(binsAll(6).bins), (r(runTimes>0))',runTimes(runTimes>0)'))=1;

binsAll(1).parNum = [n_pos_bins(1).^2 0 0 0];
binsAll(2).parNum = [n_pos_bins(2).^2 0 0 0];
binsAll(3).parNum = [n_pos_bins(3).^2 0 0 0];
binsAll(4).parNum = [0 n_dir_bins 0 0];
binsAll(5).parNum = [0 0 n_speed_bins 0];
binsAll(6).parNum = [0 0 0 n_run_bins];

% establish all of the models
for ni = 1:size(S,1)
 disp(['condition ' num2str(recordingLoc) ', bird ' (num2str(b)) ', session ' num2str(s)])
    initCon = -2; % guess for the flat model
    model=struct;
 curr = 1;
    S_Neuron = S(ni,:)';
    fprintf('\t- Fitting Neuron %d of %d\n', ni, size(S,1));
    disp('find best single variable model')
% fit each of the positional models and take the best one. this is now
% position
A = cell(numel(binsAll),1);

for n = 1:6
    A{n} = binsAll(n).bins;
end
% fit the null model
[model(curr).param,model(curr).loglik] = fit_model(ones(size(S_Neuron)),S_Neuron,[0 0 0 0],numFolds,initCon);
model(curr).pToMeanFiring = nan;
model(curr).modelType = 'null model';
model(curr).medianImprovement = 0;
curr = curr+1;
%

% fit the spatial models
for n = 1:3
        fprintf('\t- Fitting model %d of %d\n', n, numel(A));
        [model(curr).param,model(curr).loglik] = fit_model(A{n},S_Neuron,binsAll(n).parNum,numFolds,model(1).param);
        model(curr).pToMeanFiring = signrank(model(1).loglik,model(curr).loglik,'tail','right');
        model(curr).modelType = binsAll(n).label;
           imp = model(1).loglik-model(curr).loglik;
     model(curr).medianImprovement = mean(imp);
        curr = curr+1;
end

%% find the simplest spatial component
[~,SM] = max(vertcat(model(2:4).medianImprovement));
disp(['best fit spatial scale =' num2str(n_pos_bins(SM))])
for n = 4:numel(A)
    fprintf('\t- Fitting 1 Par model %d of %d\n', n, numel(A));
     [model(curr).param,model(curr).loglik] = fit_model(A{n},S_Neuron,binsAll(n).parNum,numFolds,model(1).param);
     model(curr).pToMeanFiring = signrank(model(1).loglik,model(curr).loglik,'tail','right');
     model(curr).modelType = binsAll(n).label;
        imp = model(1).loglik-model(curr).loglik;
     model(curr).medianImprovement = mean(imp);
    curr = curr+1;
end
%%
% find the best model
inContention = [1 SM+1 5:7];
[~,best1M] = max([model(inContention).medianImprovement]);
%%
inContention = inContention(2:end);
if best1M~=1
    best1M = best1M-1;
    disp('find best double variable models')
binsCell = binsAll(inContention-1);

%% find the pairs of best1M
notBest = setdiff(1:4,best1M);
%
L1 = sort([best1M notBest(1)]);
L2 = sort([best1M notBest(2)]);
L3 = sort([best1M notBest(3)]);
A = {};
A{1} = [binsCell(L1).bins];
A{2} = [binsCell(L2).bins];
A{3} = [binsCell(L3).bins];
PN = {};
PN{1} = sum(vertcat(binsCell(L1).parNum));
PN{2} = sum(vertcat(binsCell(L2).parNum));
PN{3} = sum(vertcat(binsCell(L3).parNum));

modelName{1} = [binsCell(L1).label];
modelName{2} = [binsCell(L2).label];
modelName{3} = [binsCell(L3).label];
%%
for n = 1:numel(A)
    fprintf('\t- Fitting 2 Par model %d of %d\n', n, numel(A));
     [model(curr).param,model(curr).loglik] = fit_model(A{n},S_Neuron,PN{n},numFolds,model(1).param);
     model(curr).pToMeanFiring = signrank(model(1).loglik,model(curr).loglik,'tail','right');
     model(curr).modelType = modelName{n};
     imp = model(1).loglik-model(curr).loglik;
     model(curr).medianImprovement = mean(imp);
    curr = curr+1;
end
%%
% compare to the best single model and take the
% best one. if it is the 2 var model, stop
inContention2 = [inContention(best1M) 8:10];
[~,best2M] = max([model(inContention2).medianImprovement]);

% sig = signrank(model(inContention(best1M)).loglik,model(inContention2(best2M)).loglik,'tail','right');
% %%
    if best2M~=1 % fit the 2 triple models. compare to best double
      disp('find best triple variable models')
    notBest2 = setdiff(1:4,[best1M notBest(best2M-1)]);

    L1 = sort([best1M notBest(best2M-1) notBest2(1)]);
    L2 = sort([best1M notBest(best2M-1) notBest2(2)]);
A = {};
    A{1} = [binsCell(L1).bins];
    A{2} = [binsCell(L2).bins];
PN = {};
PN{1} = sum(vertcat(binsCell(L1).parNum));
PN{2} = sum(vertcat(binsCell(L2).parNum));
    modelName{1} = [binsCell(L1).label];
    modelName{2} = [binsCell(L2).label];
%%
    for n = 1:numel(A)
        fprintf('\t- Fitting 3 Par model %d of %d\n', n, numel(A));
         [model(curr).param,model(curr).loglik] = fit_model(A{n},S_Neuron,PN{n},numFolds,model(1).param);
         model(curr).pToMeanFiring = signrank(model(1).loglik,model(curr).loglik,'tail','right');
         model(curr).modelType = modelName{n};
            imp = model(1).loglik-model(curr).loglik;
     model(curr).medianImprovement = mean(imp);
        curr = curr+1;
    end
          %%
    inContention3 = [inContention2(best2M) 11 12];
[~,best3M] = max([model(inContention3).medianImprovement]);
    %best3M = inContention3(best3M)
%
%    sig = signrank(model(inContention2(best2M)).loglik,model(inContention3(best3M)).loglik,'tail','right');
        if best3M~=1 % fit the full model.
         A = horzcat(binsCell.bins);
         PN = sum(vertcat(binsCell(L2).parNum));
         disp('fitting full model')
         [model(curr).param,model(curr).loglik] = fit_model(A,S_Neuron,PN,numFolds,model(1).param);
         model(curr).pToMeanFiring = signrank(model(1).loglik,model(curr).loglik,'tail','right');
         model(curr).modelType = 'full'; 
            imp = model(1).loglik-model(curr).loglik;
     model(curr).medianImprovement = mean(imp);
                  
        if model(inContention3(best3M)).medianImprovement< model(curr).medianImprovement
         model(curr).bestModel = model(curr).modelType;
        model(curr).bestModelNumber = [1 2 3 4];
         
        else
            model(curr).bestModel = model(inContention3(best3M)).modelType;
            model(curr).bestModelNumber = sort([best1M notBest(best2M-1) notBest2(best3M-1)]);
        end
            
        else % 3 parameter model is best
         model(curr).bestModel = model(inContention2(best2M)).modelType;
         model(curr).bestModelNumber = sort([best1M notBest(best2M-1)]);
        end
    else %2 
       
        model(curr).bestModel = model(inContention(best1M)).modelType;
        model(curr).bestModelNumber = best1M;
    end
    
else %1 parameter model is best
          model(curr).bestModel = model(1).modelType;
        model(curr).bestModelNumber = [];
    
end
disp(['best model was ' model(curr).bestModel])
disp (' ')
% after running each neuron save
modelAll(b,recordingLoc,s,ni).model = model;
save(fp,'modelAll','-v7.3')

end
end
end
end

% helper function to assign the bins for dash modulation
function binAssign = speedCrossingBin(hopThresh,speed,nbins)

%find speedCrossings
low = speed<hopThresh;
high = speed>hopThresh;
crossing = low(1:end-1) & high(2:end);
range = 1:length(crossing);
frameCross = range(crossing==1);

for i = 1:length(frameCross)-1
      j = i+1;
      if ~isnan(frameCross(j))
  
        while j<=length(frameCross) && (frameCross(j)-frameCross(i))<20
            frameCross(j) = nan;
            j = j+1;
        end
      end 
end
frameCross(isnan(frameCross))= [];

% make the bin assignment
nearest = cumsum(sum((1:length(speed)) - repmat(frameCross',1,length(speed))==0));
lengNearest = histcounts(nearest,.5:1:(length(frameCross)+.5));
tBefore = sum(nearest==0); 

tOfRun = [nan(1,tBefore) repelem(frameCross,lengNearest)];
frameToRun = (1:length(speed))-tOfRun;

binEdges = round([0 cumsum(10.^linspace(-1,0.5,nbins-1)) inf]*20);
[~,~,binAssign] = histcounts(frameToRun,binEdges);
end