function figureS4(data,condition)
% generate figureS4
%
%   data: the physiology data structure
%   condition: specify which of the recording conditions (locations) to
%   plot
% condition 1 == aDL
% condition 2 == pDL
% condition 3 == aHp
% Marissa Applegate, Aronov Lab 2022

sigThresh = .01;
nshuff = 500;
for b = 1:4 %for each bird
    Parameter = data(b,condition).Parameter;
    Tuning = data(b,condition).Tuning;
    
    spat = vertcat(Parameter.pSpatial);
    stab = vertcat(Parameter.pStability);
    
    fieldsAll = vertcat(Tuning.fieldSegmentation);

    %calculte the number of fields
    resh = reshape(fieldsAll,size(fieldsAll,1),[]);
    nf = max(resh,[],2);
    
    maps = vertcat(Tuning.spatialMap);

    incl = spat<sigThresh & stab<sigThresh & nf>=4;
    fieldsAll = fieldsAll(incl,:,:);
    maps = maps(incl,:,:);
    nf = nf(incl);

    COMx = nan(sum(incl),6);
    COMy = nan(sum(incl),6);
    d = nan(sum(incl),6);
    
    % for each cell get the COM
    for ni = 1:sum(incl)
        neuronField = squeeze(fieldsAll(ni,:,:));
        map = squeeze(maps(ni,:,:));
        for f = 1:nf(ni)
            fieldMask = neuronField==f;
            [COMx(ni,f),COMy(ni,f)] = getCOM(fieldMask,map);
        end
    d(ni,1:f) = calculateDistances([COMx(ni,1:f)', COMy(ni,1:f)']);
    end

    % plot each cell
    for i = 1:sum(incl)

    cov = std(d(i,1:nf(i)))/mean(d(i,1:nf(i)));
    % draw from uniform distribution
    covUniform = zeros(1,nshuff);
    for ii = 1:nshuff
    covUniform(ii) = drawCOVFromRand(nf(i));
    end

    % make the figures
    r = 0:.05:1;
    figure
    subplot(1,3,1)
    m = squeeze(maps(i,:,:));
    imagesc(m)
    axis square
    axis off
    colormap('jet')
    caxis([0 prctile(m(:),99)])
    set(gca,'xdir','reverse')
    subplot(1,3,2)
    imagesc(squeeze(fieldsAll(i,:,:)))
    axis square
    axis off
    colormap('jet')
    set(gca,'xdir','reverse')

    subplot(1,3,3)
    histogram(covUniform,r,'normalization','probability','faceColor',[.5 .5 .5])
    hold on

    plot([cov cov],[0 .2],'-r','lineWidth',1)
    xlabel('CV')
    ylabel('fraction of shuffled dist.')
    box off

    end
end
end

function [COMx,COMy] = getCOM(field,map)
[X,Y] = meshgrid(1:40,1:40);
X = X(field); Y = Y(field);

firing = map(field);
normFiring = firing/sum(firing);
COMx = sum(X.*normFiring); COMy = sum(Y.*normFiring);
end

function cov = drawCOVFromRand(n)
goodDraw = false;
while ~goodDraw
coords = rand(n,2).*40;
    
% calculate distances
d = calculateDistances(coords); 

if sum(d>13)==length(d)
goodDraw = true;
end

end
% calculate covariance
cov = std(d)/mean(d);
end

function d = calculateDistances(coords)
[p1,p2]= meshgrid(1:size(coords,1));
keep = triu(true(size(coords,1)),1);
pairs = [p1(keep),p2(keep)];

coords1 = coords(pairs(:,1),:);
coords2 = coords(pairs(:,2),:);

dAll = sqrt(sum((coords1-coords2).^2,2));

% for each coordinate, find the two closest neighbors and average
d = zeros(size(coords,1),1);
for i = 1:size(d)
inContention = sum(pairs==i,2)==1;
dConsider = sort(dAll(inContention));

d(i) = mean(dConsider(1:2));   
end
end