function figureS5(data,condition)
% generate figureS5
%
%   data: the physiology data structure
%   condition: specify which of the recording conditions (locations) to
%   plot
% condition 1 == aDL
% condition 2 == pDL
% condition 3 == aHp
% Marissa Applegate, Aronov Lab 2022


d = zeros(4,40,40);
fAll = zeros(0,40,40);
nfAll = 0;

% get all of the observed densities for each bird  
for b = 1:4
    Parameter = data(b,condition).Parameter;
    Tuning = data(b,condition).Tuning;
    fields = vertcat(Tuning.fieldSegmentation);
    
    %calculte the number of fields
    resh = reshape(fields,size(fields,1),[]);
    nf = max(resh,[],2);
    
    spat = vertcat(Parameter.pSpatial);
    stab = vertcat(Parameter.pStability);
    
    incl = spat<.01 & stab<.01;
    
    fields = fields(incl,:,:);
    fAll = cat(1,fAll,fields);
    nf = sum(nf(incl));
    nfAll = nfAll+ nf;
    fields = fields>0;
    d(b,:,:) = squeeze(nansum(fields,1))./nf;

end
d = squeeze(mean(d,1)); %image this variable for figure A

%calculate the distance to the nearest corner
[x,y] = meshgrid(0:39,0:39);
dis = sqrt(x.^2 + y.^2);
d2 = fliplr(dis);
d3 = flipud(dis);
d4 = flipud(d2);
dAll = cat(3,dis,d2,d3,d4);
dis = min(dAll,[],3);

r = 0:1:25;

density = zeros(length(r)-1,1);
for i = 1:length(density)
incl = dis>r(i) & dis<=r(i+1);
    dIncl = d(incl);
    density(i) = nanmean(dIncl);
end

%find the null distribution
fieldNull = findNullDistribution(fAll);

%convert from pixels to cm
pix = 91/40;

figure
hold on
c = {'r','b','k'};
plot(r(1:end-1)*pix,density,c{condition},'lineWidth',1)
high = prctile(fieldNull,99);

plot([0 r(end-1)*pix],[high high],'color','k')
ylim([0 .2])
xlim([0 r(end-1)]*pix)
xlabel('distance to corner (cm)')
ylabel('fraction of fields')
end

%%
function fieldNull = findNullDistribution(fields)
nAll = [];

for f = 1:size(fields,1)
    map = squeeze(fields(f,:,:));
   for f2 = 1:max(map(:))
       n = sum(map(:)==f2);
       nAll = [nAll; n];
   end
end

nAll = nAll/(40^2); %probability of a field hitting a pixel
bt = bootstrp(10000,@sum,nAll); % bootstrap over observed fields
fieldNull = bt/length(nAll); % get the field density
end